<?php

/**
 * Plugin Name: Carerix
 * Plugin URI: https://development.wiki.carerix.com/cxwiki/doku.php?id=cxwordpress2
 * Description: Autopublish vacancies you created in Carerix to your website and receive applications from jobseekers.
 * Version: 3.0.3
 * Requires at least: 5.2
 * Requires PHP: 7.4
 * Author: Carerix
 * Author URI: http://www.carerix.com
 */

require_once 'Constants.php';
require_once 'vendor/Transliteration/JTransliteration.php';
require plugin_dir_path( __FILE__ ) . 'vendor/autoload.php'; // autoloader for composer

use Carerix\Api\Rest\Client;
use Carerix\Api\Rest\Entity\CRPublication;
use Carerix\Api\Rest\Manager;
use Carerix\Api\Rest\Entity;

/**
 * Carerix class
 * Used to set up the running environment of the plugin
 */
class Carerix {

	const VERSION = '3.0.3';
	const REMOTE_PATH = 'https://tools.carerix.com/wpupdate';

	private $db = null;

	/**
	 *
	 * @var WP_Interfacer
	 */
	private $interfacer = null;
	private $credentialsError = false;
	static $JOB_NOT_FOUND = false;
	static $DISPLAYED = false;
	static $TMPDIR = '';

	/**
	 * @param wpdb Wordpress Database Object
	 */
	public function __construct($db) {
		$this->db = $db;

		$base_directory = dirname(__FILE__);
		self::$TMPDIR = $base_directory . DIRECTORY_SEPARATOR . 'tmp';

//		$loader = new Loader($base_directory);
//		$loader->run();
	}

	/**
	 * Starts the plugin main execution thread and registers
	 * the actions for the activate and init hooks
	 */
	public function run() {
		// Initialize the Wordpress Interfacer
		$this->interfacer = WP_Interfacer::getInstance($this->db);

		$this->bootStrapCarerixApi();

		// add_action('init', array($this, 'session_init'));
		// $this->interfacer->on_action('init', array($this, 'process_post'));
		$this->interfacer->on_activate(array($this, 'upgrader_process_complete'));
		$this->interfacer->on_init(array($this, 'detectArea'));

		$this->registerCron();
		add_action('widgets_init', array($this, 'registerWidgets'));
		add_action('after_setup_theme', array($this, 'disableCaching'));
		add_action('init', array($this, 'activate_au'));
		add_action('init', array($this, 'set_apply_cookies'));
		add_action('admin_init', array($this, 'cx_verify_requirements'));

		if (isset($_GET['logout'])) {
			\Carerix\Cookie\Cookie::deleteCookie("captcha");
			\Carerix\Cookie\Cookie::deleteCookie("logged_in");
			\Carerix\Cookie\Cookie::deleteCookie("employee_id");
			\Carerix\Cookie\Cookie::deleteCookie("kept");
			\Carerix\Cookie\Cookie::deleteCookie("name");
			\Carerix\Cookie\Cookie::deleteCookie("sent");
		}

		//$this->interfacer->taxonomy_clear_cache('category');
	}

	public function do_feed_custom() {
		global $wp;

		// get all publications from categories of sources
		// get the sources

		$sources = Entity_Source_Job::getAll();
		$categoryIDs = array();
		foreach ($sources as $source) {
			$sourceCron = new Entity_Cron_Job($source);
			$categoryName = $sourceCron->getCategory();
			if ($categoryName !== FALSE) {
				array_push($categoryIDs, get_cat_ID($categoryName));
			}
		}

		query_posts(array('category__and' => $categoryIDs, 'posts_per_page' => 10));
		load_template(ABSPATH . 'wp-includes\feed-rss.php');
	}

	/**
	 * Schedules the custom cron mechanism
	 */
	public function registerCron() {
		$this->interfacer->add_cron_schedule(array($this, 'carerix_add_custom_cron_schedule'));

		// sources cron
		$this->interfacer->schedule_next_cron('carerixinterval', 'cron_sync');
		$this->interfacer->on_action('cron_sync', array($this, 'cron_sync'));

		// application queue cron (handles (re)submitting application-forms to the REST-API
		$this->interfacer->schedule_next_cron('carerixapplicationqueue', 'applicationqueue_manager_cron');
		$this->interfacer->on_action('applicationqueue_manager_cron', array($this, 'applicationqueue_manager_cron'));

		// report cron
		$this->interfacer->schedule_next_cron('daily', 'report_cron');
		$this->interfacer->on_action('report_cron', array($this, 'report_cron'));

		// clean taxonomies cron
		$this->interfacer->schedule_next_cron('hourly', 'clean_taxonomies_cron');
		$this->interfacer->on_action('clean_taxonomies_cron', array($this, 'clean_taxonomies_cron'));

		// attachment cleanup cron
		$this->interfacer->schedule_next_cron('hourly', 'cleanup_attachments_cron');
		$this->interfacer->on_action('cleanup_attachments_cron', array($this, 'cleanup_attachments_cron'));

		$this->interfacer->on_action('admin_notices', array($this, 'carerix_notice'));
	}

	/**
	 * Application queue cron (handles (re)submitting application-forms to the REST-API
     *
     * */
	public function applicationqueue_manager_cron() {
		$cronRunner = new Environment_CronRunner();
        $cronRunner->runForPendingApplicationSubmissions();
    }

	public function clean_taxonomies_cron() {
		$this->clear_taxonomies();
	}

	public function carerix_notice() {
		global $pagenow;

		$params = array();

		parse_str($_SERVER['QUERY_STRING'], $params);
		if (isset($params['page']) && stripos($params['page'], 'carerix') === 0 || $pagenow == 'index.php') {
			$this->check_sync_status();
			$this->check_environment_status();
		}
	}

	public function check_environment_status() {
		$isSane = Environment_Installation::isSane();
		if ($isSane === false) {
			$paths = Environment_Installation::isSane(true);
			if (!empty($paths)) {
				$_p = '<ul>';
				foreach ($paths as $path) {
					$_p .= "<li>$path</li>";
				}
				$_p .= '</ul>';
				echo "<div class='error'><p>Carerix Warning : The following directories are not writeable by PHP</p> $_p <p>Please make sure they have the right permissions set.</p></div>";
			}
		}
	}

	public function check_sync_status() {
		$settingsTable = new WP_Table(Environment_Installation::TABLE_SETTINGS);

		$valuesInDB = $settingsTable->getByKey('cron_status');
		if (count($valuesInDB) < 1)
			return;

		$status = $valuesInDB[0]['value'];

		$valuesInDB = $settingsTable->getByKey('cron_time');
		if (count($valuesInDB) < 1)
			return;

		$lasSynctime = (int) $valuesInDB[0]['value'];

		if ($status == 'ok')
			return;

		$currentTime = time();
		if ($currentTime - $lasSynctime > WP_Constants::SYNC_WARNING_TIME) {
			echo "<div class='error'><p>Carerix Warning : the automatic syncronization process failed. Please try manual syncronization. If the problem persists please contact your Carerix administrator</p></div>";
		}
	}


	public function add_head() {
		$html = '';
		$html .= '<meta name="carerix_version" content="v' . Carerix::VERSION . '">';
		$html .= Environment_Settings::getFormConversionGeneral();
		$html .= '<link href="//netdna.bootstrapcdn.com/font-awesome/4.0.1/css/font-awesome.css" rel="stylesheet">';
		echo $html;
	}


	/**
	 * Main hook method which is called by the cron schedular or manually by https://yoursite.com/?carerix_sync
	 *
	 * @param bool $manual_called
	 */
	public function cron_sync($manual_called = false)
	{
		/**
		 * Flush (Redis) caching which may cause sync problems
		 */
		global $wp_object_cache;
		$wp_object_cache->flush(0);

		do_action( 'cxwordpress_sync_cron_start' );

		set_time_limit(0); // normally, the cron already has time limit 0, but just to make sure
		if ($manual_called)
		{
			ini_set('output_buffering', 'off');
			ini_set('zlib.output_compression', false);
			while (@ob_end_flush()) ;

			ini_set('implicit_flush', true);
			ob_implicit_flush(true);
			echo "<!DOCTYPE html>\n<html><body>\n";
			echo "Working.. Please wait for the finish message.";
			echo str_pad("", 1024, " ");
			echo "<br />\n";
			@ob_flush();
			flush();
		}
		// if lock exists, return
		/*
		  if(Environment_Settings::isTheCronLocked()) {
		  return;
		  }

		  Environment_Settings::lockTheCron(); */
		wp_suspend_cache_invalidation(true);
		$this->interfacer->taxonomy_clear_cache('category');
		/*
		 * when running a cron, wordpress does not include all required files
		 * for what the plugin needs to do.
		 * This is where to add extra requires
		 */

		$taxonomyImportFile = implode(DIRECTORY_SEPARATOR, array(ABSPATH, 'wp-admin', 'includes', 'taxonomy.php'));
		require_once $taxonomyImportFile;

		$cronRunner = new Environment_CronRunner();
		$result = $cronRunner->run($manual_called); // This runs the actual sync job in wp-content/plugins/CxWordpress/Environment/CronRunner.php


		$settingsTable = new WP_Table(Environment_Installation::TABLE_SETTINGS);

		$status = ($result) ? 'ok' : 'nok';

		// update cron status

		$data = array(
			'key'   => 'cron_status',
			'value' => $status,
		);

		$valuesInDB = $settingsTable->getByKey('cron_status');
		if (count($valuesInDB) >= 1)
		{
			$record = $valuesInDB[0];
			$settingsTable->update($record['id'], $data);
		}
		else
		{
			$settingsTable->insert($data);
		}

		// set the cron time
		$time = time();

		$data = array(
			'key'   => 'cron_time',
			'value' => $time,
		);

		$valuesInDB = $settingsTable->getByKey('cron_time');
		if (count($valuesInDB) >= 1)
		{
			$record = $valuesInDB[0];
			$settingsTable->update($record['id'], $data);
		}
		else
		{
			$settingsTable->insert($data);
		}

		wp_cache_delete( 'restapi_one_time', CARERIX_CACHE_GROUP);
        wp_suspend_cache_invalidation(false);
		if ($manual_called)
		{
			echo "\nProcess Finished";
			echo "\n</body></html>";
			@ob_flush();
			flush();
		}

		do_action( 'cxwordpress_sync_cron_finish', $status ); // status= ok|nok
		// Environment_Settings::unlockTheCron();
	}

	public function report_cron($mode = 'cron') {
		$reporting = 'on';
		// If reporting is not approved by the user, exit early
		if ($reporting !== 'on') {
			return;
		}

		$data = array();
		$data['report_mode'] = $mode;
		$data['php_version'] = PHP_VERSION;
		$data['home_url'] = home_url();
		$data['site_url'] = site_url();
		$data['active_plugins'] = get_option('active_plugins');
		$data['wordpress_version'] = get_bloginfo('version');

		$curl_channel = curl_init();
		curl_setopt($curl_channel, CURLOPT_URL, "https://stima.carerix.com/wp_report/report.php");
		curl_setopt($curl_channel, CURLOPT_POST, count($data));
		curl_setopt($curl_channel, CURLOPT_SSL_VERIFYHOST, 0);
		if (version_compare(phpversion(), '7.1', '>=')) {
			/** @noinspection PhpElementIsNotAvailableInCurrentPhpVersionInspection */
			curl_setopt($curl_channel, CURLOPT_SSL_VERIFYPEER, false);
		}
		curl_setopt($curl_channel, CURLOPT_POSTFIELDS, http_build_query($data));

		$server_response = curl_exec($curl_channel);
		curl_close($curl_channel);
	}

	/**
	 * Adds a new schedule interval to the Wordpress cron schedules
	 *
	 * @param array $schedules
	 * @return array
	 */
	public function carerix_add_custom_cron_schedule($schedules) {
		$interval_in_seconds = 600;
		$interval_in_minutes = $interval_in_seconds / 60;
		$text = 'Every ' . $interval_in_minutes . ' minutes';
		$schedules['carerixinterval'] = array(
			'interval' => $interval_in_seconds,
			'display' => __($text),
		);

		$interval_in_seconds = 120;
		$interval_in_minutes = $interval_in_seconds / 60;
		$text = 'Every ' . $interval_in_minutes . ' minutes';
		$schedules['carerixapplicationqueue'] = array(
			'interval' => $interval_in_seconds,
			'display' => __($text),
		);
		return $schedules;
	}

	/**
	 * Retrieves the AppName and AppToken from the database and tries to setup
	 * the Entity manager.
	 */
	public function bootStrapCarerixApi() {
		$app_name = Environment_Installation::getAppName();
		$app_token = Environment_Installation::getAppToken();

		$input_data = filter_input_array(INPUT_POST, FILTER_SANITIZE_FULL_SPECIAL_CHARS);
		if (isset($input_data['app_name'])) {
			$app_name = $input_data['app_name'];
			$app_token = $input_data['app_token'];
		}

		try {
			$client = new Client();
			$manager = new Manager($client);

			$manager->autoDiscoverEntities();
			$manager->setUrl(WP_Constants::CARERIX_API_ENDPOINT);
			$manager->setUsername($app_name);
			$manager->setPassword($app_token);

			Entity::setManager($manager);

			// try to fetch one medium to check if authentication works
			/*
			  $params = array(
			  'start' => '0',
			  'count' => '1'
			  );
			  CRMedium::findAll($params);
			 */
		} catch (Exception $care) {
			Utils_Log::write(Utils_Log::TYPE_AUTH, false, $care);
			$this->credentialsError = true;
		}
	}

	public function queryvars($query_vars) {
		$query_vars[] = 'pub_id';
		$query_vars[] = 'candidate_id';
		$query_vars[] = 'article_id';
		$query_vars[] = 'carerix_sync';
		$query_vars[] = 'carerix_queue';
		$query_vars[] = 'carerix_sitemap';
		$query_vars[] = 'board_publication_id';
		return $query_vars;
	}


	public function the_posts($posts) {
		$foundPosts = $posts;
		$keptPosts = array();
		foreach ($foundPosts as $foundPost) {
			$toAdd = true;
			if ($foundPost->post_type == 'page') {
				$page_id = $foundPost->ID;
				$props = $this->interfacer->entity_get_properties($page_id);

				foreach ($props as $prop_key => $val) {
					if (stripos($prop_key, '_carerix_') !== FALSE) {
						$toAdd = false;
						continue;
					}
				}
			}
			if ($toAdd) {
				array_push($keptPosts, $foundPost);
			}
		}
		return $keptPosts;
	}


	/**
	 * @param WP_Query $query
	 *
	 * @return mixed|void
	 * @throws Exception
	 */
	public function pregetposts(WP_Query $query ) {
		$input_data = array();
		$input_data_post = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$input_data_get  = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);

		if ($input_data_post) {
			$input_data += $input_data_post;
		}
		if ($input_data_get) {
			$input_data += $input_data_get;
		}

		// Redirect something like https://site.com/?pub_id=512 to a pretty (post) URL
		// Also take other/alias publication-IDs in account which are published to different mediums
		if (isset($query->query_vars['pub_id'])) {
			// Extract query string to forward to perm URL. This way parametrised vacancy landingpages will stay appreciated
			$get_vars = $_GET;
			unset($get_vars['pub_id']);
			$new_query_string = http_build_query($get_vars);
			$new_query_string = strlen($new_query_string) ? ("?" . $new_query_string) : "";

			$pub_id = (int) $query->query_vars['pub_id'];
			$gotSomething = false;
			$ents = $this->interfacer->entities_get_where_aliaspubid($pub_id);
			if ( count($ents) == 1) {
				/** @var WP_Post $post */
				$post = $ents[0];
				$gotSomething = $post->ID;
			} else {
				$ents = $this->interfacer->entities_get_where_property('_carerix_job', $pub_id);
				foreach ($ents as $ent) {
					$gotSomething = $ent->post_id;
					$perm = $this->interfacer->get_post_permalink($gotSomething);
					if ($perm) break;
				}
			}

			if ($gotSomething) {
				$perm = $this->interfacer->get_post_permalink($gotSomething) . $new_query_string;
				header("Location: $perm");
				die();
			} else {
				self::$JOB_NOT_FOUND = true;
			}
		}

		if (isset($query->query_vars['article_id'])) {
			$arts = $this->interfacer->entities_get_where_property('_carerix_article', $query->query_vars['article_id'], FALSE);
			$gotSomething = false;
			foreach ($arts as $art) {
				$gotSomething = $art->post_id;
				break;
			}
			if ($gotSomething) {
				$perm = $this->interfacer->get_post_permalink($gotSomething);
				header("Location: $perm");
				die();
			}
		}

		if (isset($query->query_vars['candidate_id'])) {
			$ents = $this->interfacer->entities_get_where_property('_carerix_candidate', $query->query_vars['candidate_id']);
			$gotSomething = false;
			foreach ($ents as $ent) {
				$gotSomething = $ent->post_id;
				break;
			}
			if ($gotSomething) {
				$perm = $this->interfacer->get_post_permalink($gotSomething);
				header("Location: $perm");
				die();
			}
		}

		if (isset($query->query_vars['carerix_sync'])) {
			remove_all_actions('add_post_meta');
			remove_all_actions('update_post_meta');
			//$this->list_hooked_functions();

			$this->cron_sync(true);
			wp_die("", "Carerix sync", 200);
		}

		if (isset($query->query_vars['carerix_queue'])) {
            echo "Carerix Queue<br><br>";

			$submissionqueue = new Carerix\CxWordpress\Applicationsubmissionqueue();
			$filledQueue = $submissionqueue->loadPendingApplicationSubmission( true );

			if ( $filledQueue ) {
					//Entity_Form_EmployeeApply::submitValidatedApplicationToCarerix($submissionqueue->getFormData(), $submissionqueue->getPublicationId(), $submissionqueue->getStoredHashKey());
					// $submissionqueue->deleteProcessedApplicationSubmission();
				echo "Found one or more items in the applcation submission queue.<br>";
			}

			wp_die("", "Carerix queue", 200);
		}

		if (isset($query->query_vars['carerix_sitemap']))
		{
			$args = array (
				'post_type'     => 'post',
				'numberposts'   => -1,
				'order'         => 'DESC',
				'meta_key'      => 'managed_by',
				'meta_value'    => WP_Constants::META_KEY_PREFIX
			);
			$posts = get_posts($args);

			$xml_sitemap = '<?xml version="1.0" encoding="UTF-8"?>' . "\n" . '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
			foreach($posts as $post) {
				$postID = $post->ID;
				$permalink = get_permalink($postID);
				$lastmod = date(DATE_W3C, strtotime($post->post_date));
				$xml_sitemap .= "  <url>\n";
				$xml_sitemap .= "    <loc>{$permalink}</loc>\n";
				$xml_sitemap .= "    <lastmod>{$lastmod}</lastmod>\n";
				$xml_sitemap .= "  </url>\n";
			}
			$xml_sitemap .= "</urlset>";

			header("Content-type: application/xhtml+xml");
			echo $xml_sitemap;
			die();
		}


		if (!empty($input_data) && isset($input_data['carerix_search_action']) && ( $query->is_main_query() && ( $query->is_page() || $query->is_search() || $query->is_category() || $query->is_archive() ) ) ) {
			$query = apply_filters('carerix_search_action', $query);
		}

		return $query;
	}

	public function registerExtraPaths() {
		add_rewrite_rule('pubid/([0-9]+)/?$', 'index.php?&pub_id=$matches[1]', 'top');
		add_rewrite_rule('candidateid/([0-9]+)/?$', 'index.php?&candidate_id=$matches[1]', 'top');
		add_rewrite_rule('carerix_sync/([0-9]+)/?$', 'index.php?&carerix_sync=$matches[1]', 'top');
		add_rewrite_rule('board_publication_id/([0-9])/?$', 'index.php?&board_publication_id$matches[1]', 'top');
		add_rewrite_rule('article_id/([0-9]+)/?$', 'index.php?&article_id=$matches[1]', 'top');

		$this->registerTaxonomies();
	}

	/**
	 * Detects the area in which the plugin runs and registers
	 * the appropriate actions and hooks
	 */
	public function detectArea() {
		if (Environment_Settings::isPluginUpdated()) {
            $this->upgrader_process_complete();
			Environment_Settings::isPluginUpdated(true); // reset to the current new plugin version
		}

		add_feed('publications_rss', array($this, 'do_feed_custom'));
		$this->registerExtraPaths();

		add_filter('query_vars', array($this, 'queryvars'));
		add_filter('pre_get_posts', array($this, 'pregetposts'));
		add_filter('get_the_excerpt', array($this, 'get_the_excerpt'));
		add_filter('wp_doing_ajax', array($this, 'filter_wp_doing_ajax'));
		add_action('wp_ajax_cxwordpress_diagnostics', array($this, 'admin_ajax_diagnostics')); // logged in (aka admin-ajax.php)


		// Ajax action to refresh the user image
		add_action( 'wp_ajax_myprefix_get_image', 'myprefix_get_image'   );
		function myprefix_get_image() {
			if(isset($_GET['id']) ){
				$image = wp_get_attachment_image( filter_input( INPUT_GET, 'id', FILTER_VALIDATE_INT ), 'medium', false, array( 'id' => 'myprefix-preview-image' ) );
				$data = array(
					'image'    => $image,
				);
				wp_send_json_success( $data );
			} else {
				wp_send_json_error();
			}
		}


		if ($this->interfacer->user_is_authenticated() && $this->interfacer->user_is_admin()) {
			add_action( 'admin_enqueue_scripts', 'load_wp_media_files' );
			function load_wp_media_files( $page ) {
				// $currentScreen = get_current_screen();
				if ($page == "toplevel_page_carerix_general_page") { // Required for wp-content/plugins/CxWordpress/templates/admin/forms/googleforjobs_settings.php (and other pages)
					wp_enqueue_script('jquery-ui-accordion');

                    // Enqueue WordPress media scripts
					wp_enqueue_media();

					wp_enqueue_style('jquery-datatables-css', plugins_url( '/assets/styles/jquery.dataTables.min.css' , __FILE__ ));
					wp_enqueue_script('jquery-datatables-js', plugins_url( '/assets/scripts/jquery.dataTables.min.js' , __FILE__ ), array('jquery'));

					// Enqueue custom script that will interact with wp.media and other back-end functionality
					wp_enqueue_script( 'cxwordpress-admin', plugins_url( '/assets/scripts/backend.js' , __FILE__ ), array('jquery') );
				}
			}

			wp_enqueue_script('table_dnd', plugins_url('assets/scripts/jquery.tablednd.js', __FILE__), array('jquery'), null, true);

			wp_enqueue_script('hide_translations', plugins_url('assets/scripts/hide_wpml_translations.js', __FILE__));
			$this->interfacer->register_script_asset('forms.js');
			$this->interfacer->register_style_asset('backend-jquery-ui.css');
			$this->interfacer->register_style_asset('backend.css');

			wp_enqueue_style('wp-pointer');
			wp_enqueue_script('wp-pointer');

			$this->interfacer->on_admin_menu(array($this, 'createMenuEntries'));
			$this->interfacer->on_posts_columns_heads(array($this, 'add_post_columns_head'));
			$this->interfacer->on_posts_columns_contents(array($this, 'add_post_columns_content'));
			$this->interfacer->on_action('admin_head', array($this, 'meta_pointer'));

		} elseif ($this->interfacer->is_frontend()) {
			/**
			 * Cookies/sessions must be set before the browser headers are sent.
             * Because we want to user cookies in a shortcode, we must detect the
             * presence of the shortcode very early.
             * Hence the captcha creation is done here.
			 */
			add_action('wp', 'wpdocs_set_captcha_cookie_for_jobalert_shortcode');
			function wpdocs_set_captcha_cookie_for_jobalert_shortcode() {
				$post = get_post();
				if (isset($post->post_content)) {
					$hasJobAlertTag = has_shortcode($post->post_content, "cx_job_alert_subscription");
					$hasOpenApplicationTag = has_shortcode($post->post_content, "cx_open_application");

					if ( $hasJobAlertTag || $hasOpenApplicationTag )
					{
						Carerix\Captcha\Captcha::testCaptchaMatch();
						if (Environment_Settings::getApplyFormCaptchaStatus() == 'on' || Environment_Settings::getJASCaptchaStatus() == 'on') Carerix\Captcha\Captcha::buildNewCaptcha();
					}

					if ( $hasOpenApplicationTag ) {
						$pattern = get_shortcode_regex();
						if ( preg_match_all( '/'. $pattern .'/s', $post->post_content, $matches ) )
						{
							foreach ($matches[0] as $shortcode)
							{
							    if (stripos($shortcode, "[cx_open_application") !== false ) {
								    $html = do_shortcode($shortcode);
								    $post->post_content = str_replace($shortcode, $html, $post->post_content);
								    break; // One application form allowed on a single page
							    }
						    }
						}
                    }
				}
			}


			// $this->interfacer->register_script_asset('modernizr-custom.js');
			// $this->interfacer->register_script_asset('polyfiller.js');

			$this->interfacer->on_head_display(array($this, 'add_head'));
			$this->add_js_head_and_footer();

			add_filter('autoptimize_filter_js_exclude','my_ao_override_jsexclude',10,1);

            add_filter('carerix_search_action', array($this, 'carerix_search_action'));

			$this->interfacer->register_style_asset('frontend.css');

			$tellAFriend = new Frontend_Shortcode_TellAFriend();
			$this->interfacer->on_shortcode('cx_tell_a_friend', array($tellAFriend, 'run'));

			$subscription = new Frontend_Shortcode_Subscribe();
			$this->interfacer->on_shortcode('cx_job_alert_subscription', array($subscription, 'run'));

			$campaignForm = new Frontend_Shortcode_Campaign();
			$this->interfacer->on_shortcode('cx_campaign', array($campaignForm, 'run'));

			$openApplyForm = new Frontend_Shortcode_OpenForm();
			$this->interfacer->on_shortcode('cx_open_application', array($openApplyForm, 'run'));

			$applyForm = new Frontend_Shortcode_ApplyForm();
			$this->interfacer->on_shortcode('cx_apply_form', array($applyForm, 'run')); // Generates apply form. Useful for combining a vacancy-text and an apply-form on the same page. This also complies to the "direct apply" feature of Google For Jobs

			$newsletterForm = new Frontend_Shortcode_Newsletter();
			$this->interfacer->on_shortcode('cx_newsletter_subscription', array($newsletterForm, 'run'));

			$loginForm = new Frontend_Shortcode_Login();
			$this->interfacer->on_shortcode('cx_login_form', array($loginForm, 'run'));


			$jobFrontend = new Frontend_Shortcode_Job();
			$jobFrontend->register_shortcodes();

//			$this->interfacer->on_shortcode('cx_rss_for_category', array($jobFrontend, 'rss_for_category'));
//
//			$this->interfacer->on_shortcode('cx_rss_for_all', array($jobFrontend, 'rss_for_all'));
//
//			$this->interfacer->on_shortcode('cx_tell_a_friend_link', array($jobFrontend, 'tell_a_friend_link'));
//
//			$this->interfacer->on_shortcode('cx_print_link', array($jobFrontend, 'print_link'));
//
//			$this->interfacer->on_shortcode('cx_brief_section', array($jobFrontend, 'brief_section'));
//
//			$this->interfacer->on_shortcode('cx_function_group', array($jobFrontend, 'function_group'));
//
//			$this->interfacer->on_shortcode('cx_vacancy_number', array($jobFrontend, 'vacancy_number'));
//			$this->interfacer->on_shortcode('cx_vacancy_code', array($jobFrontend, 'vacancy_code'));
//			$this->interfacer->on_shortcode('cx_social_icons', array($jobFrontend, 'social_icons'));
//			$this->interfacer->on_shortcode('cx_apply_button', array($jobFrontend, 'apply_button'));
//			$this->interfacer->on_shortcode('cx_job_alert_subscription_link', array($jobFrontend, 'job_alert_subscription_link'));

			$candidateFrontend = new Frontend_Shortcode_Candidate();

			foreach (Frontend_Shortcode_Candidate::$fields as $field_key => $field_options) {
				$shortcode = 'cx_candidate_' . $field_key;
				$this->interfacer->on_shortcode($shortcode, array($candidateFrontend, 'run'));
			}

			$this->interfacer->on_frontend_content(array($this, 'frontend_content'));
		}


		$this->interfacer->on_entity_delete(array($this, 'cleanup_post'));
		$this->interfacer->on_category_delete(array($this, 'cleanup_category'));

		$this->interfacer->on_before_admin_bar_render(array($this, 'new_bar'));

		$this->interfacer->on_ajax('get_items', array($this, 'get_items'));
		$this->interfacer->on_ajax_priv('get_items', array($this, 'get_items'));

		$this->interfacer->on_ajax('get_parent', array($this, 'get_parent'));
		$this->interfacer->on_ajax_priv('get_parent', array($this, 'get_parent'));


		$this->interfacer->on_ajax_priv('sync_all', array($this, 'sync_all'));
		$this->interfacer->on_ajax_priv('sync_all_full', array($this, 'sync_all_full'));
		$this->interfacer->on_ajax_priv('reset_all', array($this, 'reset_all'));
		$this->interfacer->on_ajax_priv('synchronize_single_source', array($this, 'synchronize_single_source'));

		$this->interfacer->on_ajax_priv('save_source', array($this, 'save_source'));

		$this->interfacer->on_ajax_priv('stop_tutorial', array($this, 'stop_tutorial'));

		add_filter('found_posts', array($this, 'found_posts'));
		add_filter('loop_start', array($this, 'searchfilter'));
		add_filter('the_title', array($this, 'post_title'), 10, 2);
		add_filter('the_posts', array($this, 'the_posts'));
	}


	/**
     * Using legacy sessions during ajax calls will stale WP for 10 secs
     *
	 * @param $defined
	 *
	 * @return mixed
	 */
	public function filter_wp_doing_ajax( $defined )
	{
		/*if (is_admin())*/ session_write_close();
        return $defined;
	}

	/**
     * Endpoint wp-admin/admin-ajax.php?action=cxwordpress_diagnostics
     * URL param: section
     *
	 * @return void
	 */
	public function admin_ajax_diagnostics() {
        $request = $_GET;
		$section = sanitize_key($request['section'] ?? "nop");
		$start = intval( $request['start'] ?? 0 ); // offset in recordset (for paging)
		$length = intval( $request['length'] ?? 10 ); // number of records per 'draw'. Can be -1 to indicate that all records should be returned
        $search = sanitize_text_field( trim($request['search']['value'] ?? "") );

		$response = [];

		$items = [];
		$adminDiagnostics = new Admin_Diagnostics();
        switch ( $section ) {
	        case "general":
		        $items = $adminDiagnostics->getDiagnosticsGeneral();
		        break;
            case "medium-channels":
	            $items = $adminDiagnostics->getDiagnosticsMediumChannels();
	            break;
            case "active-publications":
                $items = $adminDiagnostics->getDiagnosticsActivePublications($request, $start, $length, $search);
                break;
        }

        if ( isset($items['data']) ) {
            // Used for case "active-publications" which uses serverside recordset processing
            $response = $items;
        } else {
	        // Add two properties to our response - 'data' and 'recordsTotal'
	        $response['data'] = !empty($items) ? $items : []; //array of item objects if we have any, otherwise an empty array
	        $response['recordsTotal'] = !empty($items) ? count($items) : 0; //total number of items without any filtering applied
        }
		$response['draw'] = isset ( $request['draw'] ) ? intval( $request['draw'] ) : 0; // for serverside recordset processing

		wp_send_json($response); //json_encodes our $response and sends it back with the appropriate headers
    }



	public function get_the_excerpt($excerpt) {
		// @todo: when on apply-form, return a better 'excerpt' based on the original post content
		return $excerpt;

		// @todo: fix no excerpt on candidates
		global $post;

		$managed_by_carerix = $this->interfacer->managed_by_carerix($post->ID);
		if ($managed_by_carerix) {
			$excerpt = do_shortcode($post->post_content);
		}
		return $excerpt;
	}

	public function found_posts($query) {

		global $wp_query;
		if (self::$JOB_NOT_FOUND && isset($wp_query->query['pub_id']) && $wp_query->query['pub_id'] !== 0) {
			// $wp_query->posts = array('[]');
			$wp_query->posts = array();
			return;
		}

		return $query;
	}

	public function post_title($title = false, $id = false) {
		if ($title === false || $id === false)
			return $title;
		$backtrace = debug_backtrace();
		foreach ($backtrace as $bkt) {
			if (isset($bkt['file'])) {
				if (basename($bkt['file']) == 'widgets.php')
					return $title;
			}
		}

		if (isset($_GET['subscribe'])) {
			if (!is_single($id))
				return $title;
			$props = $this->interfacer->entity_get_properties($id);
			foreach ($props as $prop_key => $val) {
				if (stripos($prop_key, '_carerix_') !== FALSE) {
					$labels = Utils_Labels::get();
					return $labels['CxPortal.header.jobAlertSubscription'];
					break;
				}
			}
		}
		return $title;
	}

	public function searchfilter($query) {

		if (self::$JOB_NOT_FOUND && isset($query->query_vars['pub_id']) && $query->query_vars['pub_id'] > 0) {

			$query->posts = array();
			$query->post_count = 0;
			$query->post = (object) array();
			$query->is_home = false;

			global $data;
			if (is_array($data)) {
				$data['content_length'] = 'SpecialCaseForAvadaTheme';
			}
		}

		return $query;
	}

	public function getTaxonomy($key, $section, $extra = '') {
//		$language = $this->interfacer->language_major_current();
		$language = $this->interfacer->language_major_backend_user();

		$settingName = "resource_taxonomy_{$section}_{$key}";
		if ($extra)
			$settingName .= '_' . $extra;
		$settingName .= '_' . $language;
		$settingName = str_replace('.', '_', $settingName);
		$name = Environment_Settings::_setting_get($settingName);

		if (!$name) {
			$labels = Utils_Labels::get( $language );
			$name = $labels[$key];
		}
		return array(
			'name' => "{$name}",
			'singular_name' => "{$name}",
			'search_items' => "{$name}",
			'edit_item' => "Edit {$name}",
			'update_item' => "Update {$name}",
			'add_new_item' => "Add New {$name}",
			'new_item_name' => "New {$name}",
			'menu_name' => "{$name}"
		);
	}

	public function getTaxonomySlug($key, $section, $extra = '') {
		$language = $this->interfacer->get_default_site_language();
		$settingName = "slug_taxonomy_{$section}_{$key}";
		if ($extra)
			$settingName .= '_' . $extra;
		$settingName .= '_' . $language;
		$settingName = str_replace('.', '_', $settingName);
		$slug = Environment_Settings::_setting_get($settingName);
		if (!$slug) {
			$slug = $key;
		}
		return $slug;
	}

	/**
	 * These taxonomies are displayed in the Dashboard UI in the posts-menu
	 */
	public function registerTaxonomies() {
		define("LEGACY_TAXONOMIES_SHOW_UI", true);

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_FUNCTIONGROUP, 'publications'); // TAXONOMY_FUNCTIONGROUP
		$name = $this->getTaxonomy('CxPortal.message.field.singleFunctionGroup', 'publications'); // CxPortal.header.functionGroups
		// $name["menu_name"] .= " 1";
		register_taxonomy(WP_Constants::TAXONOMY_FUNCTIONGROUP, 'post', array( // TAXONOMY_FUNCTIONGROUP_SLUG
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
			/* 'capabilities' => array( // 'manage_terms' => '', //default: manage_categories
			                        'edit_terms' => '', //default: manage_categories
			                        'delete_terms' => '', //default: manage_categories
			                        'assign_terms' => '', //default: edit_posts
            ),*/
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_FUNCTION, 'publications'); // TAXONOMY_FUNCTION_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.function', 'publications');
		// $name["menu_name"] .= " 2";
		register_taxonomy(WP_Constants::TAXONOMY_FUNCTION, 'post', array( // TAXONOMY_FUNCTION_SLUG
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_JOBGROUP, 'publications'); // TAXONOMY_JOBGROUP_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.jobGroup', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_JOBGROUP, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_SECTOR, 'publications'); // TAXONOMY_SECTOR_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.sector', 'publications'); // defined in language file
		register_taxonomy(WP_Constants::TAXONOMY_SECTOR, 'post', array( // TAXONOMY_SECTOR_SLUG
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_INDUSTRY, 'publications'); // TAXONOMY_INDUSTRY_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.industry', 'publications'); // defined in language file
		register_taxonomy(WP_Constants::TAXONOMY_INDUSTRY, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_WORKLOCATION, 'publications'); //TAXONOMY_WORKLOCATION_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.workLocation', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_WORKLOCATION, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_REGION, 'publications'); // TAXONOMY_REGION_SLUG
		$name = $this->getTaxonomy('general.message.field.region', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_REGION, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_COUNTRY, 'publications'); // TAXONOMY_COUNTRY_SLUG
		$name = $this->getTaxonomy('general.message.field.country', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_COUNTRY, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_EDUCATION, 'publications'); // TAXONOMY_EDUCATION_SLUG
		$name = $this->getTaxonomy('general.header.education', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_EDUCATION, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_WORKLEVEL, 'publications'); // TAXONOMY_WORKLEVEL_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.professionalLevel', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_WORKLEVEL, 'post', array( // TAXONOMY_WORKLEVEL_SLUG
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false
			)
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_PROCEDURE, 'publications'); // TAXONOMY_PROCEDURE_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.procedure', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_PROCEDURE, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_BUSINESSLINE, 'publications'); // TAXONOMY_BUSINESSLINE_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.businessLine', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_BUSINESSLINE, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_CONTRACTTYPE, 'publications'); // TAXONOMY_CONTRACTTYPE_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.contractType', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_CONTRACTTYPE, 'post', array(
			'show_ui' => LEGACY_TAXONOMIES_SHOW_UI,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_HOURSPERWEEK, 'publications'); // TAXONOMY_HOURSPERWEEK_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.weekHours', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_HOURSPERWEEK, 'post', array(
            'show_ui' => false,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_MINSALARY, 'publications'); // TAXONOMY_MINSALARY_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.minSalary', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_MINSALARY, 'post', array(
			'show_ui' => false,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_MAXSALARY, 'publications'); // TAXONOMY_MAXSALARY_SLUG
		$name = $this->getTaxonomy('CxPortal.message.field.maxSalary', 'publications');
		register_taxonomy(WP_Constants::TAXONOMY_MAXSALARY, 'post', array(
			'show_ui' => false,
			'hierarchical' => false,
			'labels' => $name,
			'rewrite' => array(
				'slug' => $slug,
				'with_front' => false,
				'hierarchical' => false,
			),
		));

		register_taxonomy(
			WP_Constants::TAXONOMY_PUBIDLIST,
			'post',
			array(
				'show_ui' => false,
				'label' => __( 'PubID aliases' ),
				/*'public' => false,*/
				'rewrite' => false,
				'hierarchical' => false,
			)
		);

		$additionalfieldssupport_status = Environment_Settings::getCxAdditionalFieldsSupportStatus(); // `on` or `off`
        if ($additionalfieldssupport_status == "on") {
	        $extra_fields = $this->interfacer->db_get_additional_cx_fields();
	        $extra_fields = str_replace("_", "-", $extra_fields); // change underscores in dashes for taxonomies
	        foreach ($extra_fields as $extraFieldName)
	        {
		        register_taxonomy(
			        $extraFieldName,
			        'post',
			        array(
				        'show_ui'      => true,
				        'label'        => $extraFieldName,
				        'rewrite'      => false,
				        'hierarchical' => false,
			        )
		        );
	        }
        }

//		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_CANDIDATE_LAND, 'candidate'); // TAXONOMY_CANDIDATE_LAND_SLUG
//		$name = $this->getTaxonomy('general.message.field.country', 'candidate');
//		register_taxonomy(WP_Constants::TAXONOMY_CANDIDATE_LAND, 'post', array(
//			'hierarchical' => false,
//			'labels' => $name,
//			'rewrite' => array(
//				'slug' => $slug,
//				'with_front' => false,
//				'hierarchical' => false,
//			),
//		));
//
//		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_CANDIDATE_LAND, 'candidate'); // TAXONOMY_CANDIDATE_LAND_SLUG
//		$name = $this->getTaxonomy('general.message.field.region', 'candidate');
//		register_taxonomy(WP_Constants::TAXONOMY_CANDIDATE_REGION, 'post', array(
//			'hierarchical' => false,
//			'labels' => $name,
//			'rewrite' => array(
//				'slug' => $slug,
//				'with_front' => false,
//				'hierarchical' => false,
//			),
//		));
//
//		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_1, 'candidate'); // TAXONOMY_CANDIDATE_EDUCATION_LEVEL_1_SLUG
//		$name = $this->getTaxonomy('general.header.educationLevel', 'candidate', '1');
//		register_taxonomy(WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_1, 'post', array(
//			'hierarchical' => false,
//			'labels' => $name,
//			'rewrite' => array(
//				'slug' => $slug,
//				'with_front' => false,
//				'hierarchical' => false,
//			),
//		));
//
//		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_2, 'candidate'); // TAXONOMY_CANDIDATE_EDUCATION_LEVEL_2_SLUG
//		$name = $this->getTaxonomy('general.header.educationLevel', 'candidate', '2');
//		register_taxonomy(WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_2, 'post', array(
//			'hierarchical' => false,
//			'labels' => $name,
//			'rewrite' => array(
//				'slug' => $slug,
//				'with_front' => false,
//				'hierarchical' => false,
//			),
//		));
//
//		$slug = $this->getTaxonomySlug(WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_3, 'candidate'); // TAXONOMY_CANDIDATE_EDUCATION_LEVEL_3_SLUG
//		$name = $this->getTaxonomy('general.header.educationLevel', 'candidate', '3');
//		register_taxonomy(WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_3, 'post', array(
//			'hierarchical' => false,
//			'labels' => $name,
//			'rewrite' => array(
//				'slug' => $slug,
//				'with_front' => false,
//				'hierarchical' => false,
//			),
//		));

	}

	public function save_source()
	{
		// error_reporting(0);
		check_ajax_referer('save_source', 'security'); // Added from wp-content/plugins/CxWordpress/templates/admin/settings.php --> wp_create_nonce()

		$input_data = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL) + filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);

		$response   = array();
		if (wp_verify_nonce($input_data['_carerix_nonce_field'], 'source_edit'))
		{
			switch ($input_data['type'])
			{
				case 'job':
					/**
					 * Different treatment for Sources->edit source->form. Appreciate HTML
					 *
					 * @since 2.20.1
					 */
					$input_data = filter_input_array(INPUT_GET) + filter_input_array(INPUT_POST);
					foreach ($input_data as &$input_elem) {
						$input_elem = wp_kses_post($input_elem); // Sanitize content for allowed HTML tags for post content.
					}
					$response = Admin_Sources_Jobs::ajaxSaveSource($input_data);
					break;
				case 'promotional':
					$response = Admin_Sources_PromotionalJobs::ajaxSaveSource($input_data);
					break;
				case 'candidate':
					$response = Admin_Sources_Candidates::ajaxSaveSource($input_data);
					break;
				case 'people':
					$response = Admin_Sources_People::ajaxSaveSource($input_data);
					break;
				case 'companies':
					$response = Admin_Sources_Companies::ajaxSaveSource($input_data);
					break;
				case 'newsletters':
					$response = Admin_Sources_Newsletters::ajaxSaveSource($input_data);
					break;
				default:
					break;
			}
		}
		echo json_encode($response);
		die();
	}


	/**
	 * Method called by js button at: Admin -> Carerix -> Sources -> Upon saving a source
	 *
	 * Returns json data to screen as output. To be used by ajax
	 */
	public function synchronize_single_source() {
		// error_reporting(0);
		check_ajax_referer('synchronize_single_source', 'security'); // Added from CxWordpress/templates/admin/sources/candidates_edit.php + jobs_edit.php

		ob_start(); // start being silent
		$input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$get_data   = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		if ($get_data)
		{
			$input_data += $get_data;
		}

		$response    = array();
		$hasMore     = false;
		$source_id   = (int) $input_data['id'];
		$page        = (int) $input_data['page'];
		$cronCleaner = new Utils_CronCleaner($source_id);
		if (!$page)
		{
			$cronCleaner->clean();
		}

		switch ($input_data['type'])
		{
			case 'job':
				$source                 = new Entity_Source_Job($source_id);
				$sourceData             = $source->getData();
				$sourceData['settings'] = unserialize($sourceData['settings']);

				$cronRunner = new Environment_CronRunner();
				$hasMore    = $cronRunner->runJobCronForSource($sourceData, $page);
				break;
			case 'candidate':
				$source                 = new Entity_Source_Candidate($source_id);
				$sourceData             = $source->getData();
				$sourceData['settings'] = unserialize($sourceData['settings']);

				$cronRunner = new Environment_CronRunner();
				$hasMore    = false;
				break;
			case 'promotional':
				break;
			case 'people':
				break;
			case 'companies':
				break;
			case 'newsletters':
				break;
			default:
				break;
		}
		if (!$hasMore)
		{
			$cronCleaner->run($input_data['type']);
			$post = $this->interfacer->entities_get_where_property('_carerix_job_page', $source_id);

			$post_id          = $post[0]->post_id;
			$response['post'] = $this->interfacer->get_post_permalink($post_id);
		}
		$response['hasMore']   = $hasMore;
		$response['ok']        = true;
		$response['page']      = $page;
		$response['languages'] = $this->interfacer->get_wpml_active_languages();
		ob_end_clean(); // End being silent
		echo json_encode($response);
		// launch the cleaning process;
		wp_die();
	}

	public function stop_tutorial() {
		// error_reporting(0);

		check_ajax_referer( 'stop_tutorial', 'security' );

		Environment_Settings::setStopTutorial();
		echo json_encode(array('p' => 'np'));
		die();
	}

	// Button: Dashboard->Carerix->Settings->Reset to default
	public function reset_all() {
		// error_reporting(0);

		check_ajax_referer( 'reset_all', 'security' ); // Added from wp-content/plugins/CxWordpress/templates/admin/settings.php --> wp_create_nonce()

		Environment_Installation::resetToDefault();
		echo json_encode(array('ok' => 'ok'));
		die();
	}

	/**
	 * Method called by js button at: Admin -> Carerix -> Settings -> Synchronize new items from Carerix
	 *
	 * Returns json data to screen as output. To be used by ajax
	 */
	public function sync_all()
	{
		error_reporting(0);

		check_ajax_referer( 'sync_all', 'security' ); // Added from wp-content/plugins/CxWordpress/templates/admin/settings.php --> wp_create_nonce()

		$input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL) ?? array();
		$get_data = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);

		if ($get_data) {
			$input_data += $get_data;
		}

		$tdata = array();
		$hasMore = false;

		try {
			$cronRunner = new Environment_CronRunner();
			switch ($input_data['source_type']) {
				case 'jobs':
					$hasMore = $cronRunner->runForJobs((int) $input_data['page']);
					break;
				case 'promotionals':
					// $cronRunner->runForPromotional();
					break;
				case 'candidates':
					// $hasMore = $cronRunner->runForCandidates((int) $input_data['page']);
					break;
				case 'people':
					// $cronRunner->runForPeople();
					break;
				case 'companies':
					// $cronRunner->runForCompanies();
					break;
				case 'newsletters':
					// $cronRunner->runForNewsletters();
					break;
				default:
					break;
			}
		} catch (Exception $ex) {
			$tdata['sync_error'] = true;
		}
		if (!isset($tdata['sync_error'])) {
			$tdata['sync_success'] = true;
		}
		$tdata['page'] = (int) $input_data['page'];
		$tdata['has_more'] = $hasMore;

		echo json_encode($tdata);
        wp_die();
	}

	public function sync_all_full()
	{
		$cronCleaner = new Utils_CronCleaner(0);
		$cronCleaner->clearRestApiSyncStatuses(); // Delete all sync statuses
		$this->sync_all();
	}

	public function meta_pointer() {

		echo '
            <link href="//netdna.bootstrapcdn.com/font-awesome/4.0.1/css/font-awesome.css" rel="stylesheet">
            <script>
            window.CARERIX_MESSAGE = "Are you sure you want to move away from this page without saving?";
            jQuery(document).ready(function($){
                var modal = \'<div id="modal" style="display: none; position: absolute; left: 0px; top: 0px; width:100%; height:100%;  text-align:center; z-index: 99999; background-color: rgba(1,1,1,0.5);"><div id="message" style="background: yellow; margin-top:-7px; margin-left:-7px; font-family:verdana; font-size:12px; color:#333333;  z-index: 1002;">\'+
					\'<p style="margin-top:200px; margin-left:2px; padding: 15px;" id="carerix_message">\'+window.CARERIX_MESSAGE +\'</p>\' +
                    \'<button id="proceed" class="button button-primary">Yes</button><button id="stop" class="button">No</button>\' + 
                    \'</div></div>\';
                $(modal).appendTo("body");            

                $("form.undoable :input").change(function() {
                    $("form.undoable").data("changed", true);
                });
                var clickedLink = null;
                $("a").click(function(e) {
                    if($("form.undoable").data("changed")) {
                        clickedLink = this;
                        e.preventDefault();
                        $("#modal").show();
                    }
                });
                
                $("#proceed").click(function(){
                    //$("#modal").hide();
                    window.location.href = $(clickedLink).attr("href");
                    
                });
                
                $("#stop").click(function(){ $("#modal").hide(); });
            });


            </script>';

		$params = array();
		parse_str($_SERVER['QUERY_STRING'], $params);

		if (Environment_Settings::getStopTutorial())
			return;

		echo "<script>
            options = [
                {
                    target: '#toplevel_page_carerix_general_page',
                    content: '<h3 class=\'cx_h3\'>Welcome</h3> <p>On this menu you can find every setting that you need to customize the way the plugin works</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p>',
                    position: {
                            my: 'left top',
                            at: 'left bottom',
                    },
                    close: function() {},
                    buttons: function(event, t) {
                        button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
						button.bind('click.pointer', function () {
							t.element.pointer('close');
                            carerix_pointer(1); 
						});
						return button;
                    }
                }];
            </script>";

		if (isset($params['page']) && $params['page'] == 'carerix_general_page') {
			echo "<script>
            options = options.concat([
                {
                    target: '#submit',
                    content: '<h3 class=\'cx_h3\'>Setting Credentials</h3><p>Fill in the application name add the application Token to connect with the Carerix System and click Save.</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p>',
                    position: {
                            my: 'left top',
                            at: 'left bottom',
                    },
                    
                    close: function() {},
                    buttons: function(event, t) {
                        button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
						button.bind('click.pointer', function () {
							t.element.pointer('close');
                            carerix_pointer(2); 
						});
						return button;
                    }
                },
                
                {
                    target: '#app_cron',
                    content: '<h3 class=\'cx_h3\'>Forcing the synchronize</h3><p>Have you made changes in your system and do you want them to be updated now on this website? Click on Synchronize items from Carerix to immediately syncronize with your system. Otherwise they will be updated with the automatic synchronization which takes place every 10 minutes</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p>',
                    position: {
                            my: 'left top',
                            at: 'left bottom',
                    },
                    close: function() {},
                    buttons: function(event, t) {
                        button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
						button.bind('click.pointer', function () {
							t.element.pointer('close');
                            carerix_pointer(0); 
						});
						return button;
                    }
                },
                
            ]);
            </script>";
		}

		if (isset($params['page']) && $params['page'] == 'carerix_formsgeneral_page') {
			echo "<script>
                options = options.concat([
                    {
                        target: '#addnew',
                        content: '<h3 class=\'cx_h3\'>Adding a new form</h3><p>Click Add new to start setting up an Application Form</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p>',
                        position: {
                                my: 'left top',
                                at: 'left bottom',
                        },
                        buttons: function(event, t) {
                            button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
                            button.bind('click.pointer', function () {
                                t.element.pointer('close');
                                carerix_pointer(2); 
                            });
                            return button;
                        }
                    },
                    {
                        target: '#default_form',
                        content: '<h3 class=\'cx_h3\'>Default form</h3><p>This is the default form that will be used by default when applying. This form can not be changed or removed.</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p>',
                        position: {
                                my: 'left top',
                                at: 'left bottom',
                        },
                        close: function() {},
                        buttons: function(event, t) {
                            button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
                            button.bind('click.pointer', function () {
                                t.element.pointer('close');
                                carerix_pointer(0); 
                            });
                            return button;
                        }
                    },
                ]);
                </script>";
		}

		if (isset($params['page']) && $params['page'] = 'carerix_sources_page') {
			echo "<script>
                options = options.concat([
                    {
                        target: '#addnew',
                        content: '<h3 class=\'cx_h3\'>Adding a new source</h3><p>Click Add new to start creating a new Jobs page based on Jobs feed source available in your Carerix CxTools app.</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p>',
                        position: {
                                my: 'left top',
                                at: 'left bottom',
                        },
                        buttons: function(event, t) {
                            button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
                            button.bind('click.pointer', function () {
                                t.element.pointer('close');
                                carerix_pointer(2); 
                            });
                            return button;
                        }
                    },
                    {
                        target: '#default_source',
                        content: '<h3 class=\'cx_h3\'>Default job source</h3><p>This is the default job source that was created when you provided the right credentials. All jobs existent in this source are available now in your blog.</p><p><button id=\'stop_tutorial\' class=\'button \'>Stop Tutorial</button></p> ',
                        position: {
                                my: 'left top',
                                at: 'left bottom',
                        },
                        buttons: function(event, t) {
                            button = jQuery('<a id=\"pointer-close\" style=\"margin-left:5px\" class=\"button-secondary\">Next</a>');
                            button.bind('click.pointer', function () {
                                t.element.pointer('close');
                                carerix_pointer(0); 
                            });
                            return button;
                        }
                    },
                ]);
                </script>";
		}

		$ajax_nonce_stop_tutorial  = wp_create_nonce( "stop_tutorial" );
		echo "<script>
            jQuery(document).ready( function($) {
            

                $(document).on('click', '#stop_tutorial', function(e) {  
                    var data = { action: 'stop_tutorial', security: '" . $ajax_nonce_stop_tutorial . "' };
                    $.post(ajaxurl, data, function(resp, status, jqXHR) {});
                    window._stop_tutorial = true;
                    $('.wp-pointer').hide(); 
                });

                carerix_pointer(0);
            });
        
            function carerix_pointer(index) {
                if(!window._stop_tutorial)
                    jQuery(options[index]['target']).pointer(options[index]).pointer('open');
            }

            </script>";
	}

	public function get_items() {
		$input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$parent = $input_data['parent'];
		if ($parent == '') {
			echo json_encode(array());
			die();
		}
		$table = $input_data['table'];
		echo json_encode(Entity_Form_EmployeeApply::get_system_select_options($table, $parent));
		die();
	}

	public function get_parent() {
		$input_data = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$child = $input_data['child'];
		if ($child == '') {
			echo json_encode(array());
			die();
		}

		die();
	}

	public function add_post_columns_head($defaults) {
		$currentScreen = get_current_screen();

		if( $currentScreen->id === "edit-post" ) { // Only add CX columns to Dashboard->Posts
			$defaults['carerix_pub_id'] = 'PubID';
			// $defaults['carerix_status'] = 'Sync Status'; // No interesting info
			$defaults['carerix_managed'] = 'Managed By Carerix';
		}

		return $defaults;
	}

	public function add_post_columns_content($column_name, $post_ID) {
		$input_data = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);

		if ( $column_name == 'carerix_pub_id') {
			$job_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_JOB;

			$language = '';
			if ($this->interfacer->wpml_is_active()) {
				$language = $this->interfacer->get_default_site_language();
			}

			if (isset($input_data['lang']) && $input_data['lang'] != 'all') {
				$language = trim($input_data['lang']);
			}

			if ($language) {
				$job_type .= '_' . $language;
			}

			$promotional_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_PROMOTIONAL;
			$properties = $this->interfacer->entity_get_properties($post_ID);
			$job_id = false;
			if (isset($properties[$job_type])) {
				$job_id = @$properties[$job_type][0];
			}

			if (isset($properties[$promotional_type])) {
				$job_id = @$properties[$job_type][0];
			}

			if ($job_id == false) {
				$job_id = @$properties['guid'][0];
			}
			if ($job_id) {
				echo $job_id;
			}
		}

		if ($column_name == 'carerix_status') {
			$post = $this->interfacer->entity_get($post_ID);

			$user = $this->interfacer->user_by_id($post['post_author']);
			$user_name = $user->user_login;
			if ($user_name == WP_Constants::CARERIX_USER_NICENAME) {
				$time = time();

				$lastSyncInSeconds = (int) $time - (int) strtotime($post['post_modified_gmt']);

				$base = plugins_url(basename(dirname(__FILE__)));
				$goodImgSrc = $base . '/assets/images/check.jpg';
				$badImgSrc = $base . '/assets/images/badcheck.png';

				if ($lastSyncInSeconds > WP_Constants::SYNC_WARNING_TIME) {
					echo "<img src='$badImgSrc' />";
				} else {
					echo "<img src='$goodImgSrc' />";
				}
			}
		}

		if ($column_name == 'carerix_managed') {
			$managed_by_carerix = $this->interfacer->managed_by_carerix($post_ID);

			if ($managed_by_carerix) {
				$base = plugins_url(basename(dirname(__FILE__)));
				$goodImgSrc = $base . '/assets/images/check.jpg';
				echo "<img class='managed_by_carerix' src='$goodImgSrc' />";
			}
		}
	}

	/**
	 *
	 *
	 * @global type $wp_admin_bar
	 * @global WP_Post $post
	 */
	public function new_bar() {
		global $wp_admin_bar;
		global $post;

		if ($post !== null && !( $this->interfacer->frontend_displayed_single() === FALSE && $this->interfacer->frontend_displayed_page() === FALSE)) {

			$job_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_JOB;

			if ($this->interfacer->wpml_is_active()) {
				$job_type .= '_' . $this->interfacer->get_default_site_language();
			}

			$promotional_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_PROMOTIONAL;
			$properties = $this->interfacer->entity_get_properties($post->ID);

			$job_id = false;
			if (isset($properties[$job_type])) {
				$job_id = $properties[$job_type][0] ?? null;
			}

			if (isset($properties[$promotional_type])) {
				$job_id = $properties[$job_type][0] ?? null;
			}

			if (!$job_id) {
				$job_id = $properties['guid'][0] ?? null;
			}

			if ($job_id) {
				$wp_admin_bar->add_menu(array(
					'id' => 'post_id',
					'title' => 'Carerix Publication ID: ' . $job_id,
					'href' => '#'
				));
			}
		}


		$wp_admin_bar->add_menu(array(
			'id' => 'carerix_admin_group',
			'title' => 'Carerix v' . Carerix::VERSION,
			'href' => '#'
		));


		$wp_admin_bar->add_menu(array(
			'parent' => 'carerix_admin_group',
			'id' => 'carerix_settings',
			'title' => 'Settings',
			'href' => admin_url('admin.php?page=carerix_general_page')
		));

		if (!$this->credentialsError) {
			$wp_admin_bar->add_menu(array(
				'parent' => 'carerix_admin_group',
				'id' => 'carerix_forms',
				'title' => 'Application Forms',
				'href' => admin_url('admin.php?page=carerix_formsgeneral_page')
			));

			$wp_admin_bar->add_menu(array(
				'parent' => 'carerix_admin_group',
				'id' => 'carerix_new_form',
				'title' => 'Add New Application Form',
				'href' => admin_url('admin.php?page=carerix_formsgeneral_page&tab=carerix_forms_page&action=add')
			));

			$wp_admin_bar->add_menu(array(
				'parent' => 'carerix_admin_group',
				'id' => 'carerix_sources',
				'title' => 'Sources',
				'href' => admin_url('admin.php?page=carerix_sources_page')
			));

			$wp_admin_bar->add_menu(array(
				'parent' => 'carerix_admin_group',
				'id' => 'carerix_new_source',
				'title' => 'Add New Source',
				'href' => admin_url('admin.php?page=carerix_sources_page&tab=carerix_sources_jobs_page&action=add')
			));
		}
	}

	/**
	 * Runs when the user delets a post to remove translation data from the
	 * WPML table
	 *
	 * @param int $entity_id
	 */
	public function cleanup_post($entity_id) {
		$this->interfacer->wpml_unregister_translation($entity_id);
	}

	/**
	 * Runs when the user deletes a category to remove translation data from the
	 * WPML table
	 *
	 * @param int $category_id
	 */
	public function cleanup_category($category_id) {
		if ($this->interfacer->wpml_is_active()) {
			$this->interfacer->wpml_unregister_translation($category_id);
		}
	}


	/**
	 * Runs when the user activates the plugin.
	 * It setups the tables and the environment.
	 */
	public function upgrader_process_complete() {
		$installation = new Environment_Installation();
		$is_1st_install = $installation->install();
        if ( $is_1st_install ) {
            $installation->firstInstall();
        } else {
            $installation->upgradeInstall();
        }

		$this->report_cron('activation');

		add_feed('publications_rss', array($this, 'do_feed_custom'));

		$this->registerExtraPaths();
		flush_rewrite_rules();
	}

	/**
	 * Used to register and create the pages and tabs in the admin section
	 * of wordpress
	 *
	 * @global type $menu
	 */
	public function createMenuEntries() {

// <editor-fold defaultstate="collapsed" desc="Carerix general menu and first general settings">
		$general = new Admin_General();

		$settings = new Admin_Settings();
		$general->addTab($settings);

		$import = new Admin_Conversiontracking();
		$general->addTab($import);

		$import = new Admin_Googleforjobs();
		$general->addTab($import);

		$import = new Admin_Technologypreview();
		$general->addTab($import);

		$import = new Admin_Diagnostics();
		$general->addTab($import);

		// Disabled since 2020-01-22
//    $import = new Admin_ImportExport();
//    $general->addTab($import);

		$forms = new Admin_FormsGeneral();

		$list_of_forms = new Admin_Forms();
		$form_settings = new Admin_FormsSettings();
		$forms->addTab($list_of_forms);
		$forms->addTab($form_settings);

		$sources = new Admin_Sources();

		$job_sources = new Admin_Sources_Jobs();
		$sources->addTab($job_sources);

//		$promotional_sources = new Admin_Sources_PromotionalJobs();
//		$sources->addTab($promotional_sources);

//    $candidate_sources = new Admin_Sources_Candidates();
//    $sources->addTab($candidate_sources);

//    $people_sources = new Admin_Sources_People();
//    $sources->addTab($people_sources);

//    $company_sources = new Admin_Sources_Companies();
//    $sources->addTab($company_sources);

//    $newsletter_sources = new Admin_Sources_Newsletters();
//    $sources->addTab($newsletter_sources);

		$taxonomies = new Admin_Taxonomies();

		$taxonomy_translator = new Admin_Taxonomies_Translator();
		$taxonomies->addTab($taxonomy_translator);

		$taxonomy_slugger = new Admin_Taxonomies_Slugger();
		$taxonomies->addTab($taxonomy_slugger);

		$buffers = new Admin_Buffers();

		$apply_buffers = new Admin_Buffers_Apply();
		$buffers->addTab($apply_buffers);



		$this->interfacer->add_page($general);

		if (!$this->credentialsError) {
			$this->interfacer->add_sub_page($general, $forms);
			$this->interfacer->add_sub_page($general, $sources);
			$this->interfacer->add_sub_page($general, $taxonomies);
			$this->interfacer->add_sub_page($general, $buffers);
		}

// </editor-fold>

		global $menu;
		global $submenu;
		foreach ($menu as &$menu_item) {
			if ($menu_item[2] == 'carerix_general_page') {
				$menu_item[0] = 'Carerix';
			}
		}

		if (isset($submenu['carerix_general_page'])) {
			$submenu['carerix_general_page'][5][0] = 'Carerix v' . Carerix::VERSION;
			$submenu['carerix_general_page'][5][1] = 'administrator';
			$submenu['carerix_general_page'][5][2] = '';
			$submenu['carerix_general_page'][5][3] = 'Carerix v' . Carerix::VERSION;
		}
	}

	/**
	 *
	 * Used to modify the output if the selected post is one associated
	 * with an entity in Carerix System
	 *
	 * @global WP_Post $post
	 * @param string $content
	 * @return string
	 */
	public function frontend_content($content) {
		global $post;
		global $wp_query;

		// in case of apply from jobboard, modify the content of the first
		if (!self::$DISPLAYED && self::$JOB_NOT_FOUND && isset($wp_query->query['pub_id']) && $wp_query->query['pub_id'] !== 0) {
			self::$DISPLAYED = true;
			$publicationId = $wp_query->query['pub_id'];

			$publication = CRPublication::find($publicationId, array('show' => array('publicationID', 'titleInformation')));

			if ($publication->publicationID) {
				$content = ''; // remove original content (the post)

				$content .= "<header class='entry-header jobboard'><h1 class='entry-title jobboard'>{$publication->titleInformation}</h1></header>";

				$publication_id = $publication->publicationID;
				$jobboard_form = Environment_Settings::getSelectedFormForJobboard();
				if (!$jobboard_form) {
					$default_form = Environment_Installation::getDefaultForm();
					$jobboard_form = $default_form['id'];
				}

				$form_source = new Entity_Form((int) $jobboard_form);
				$formData = $form_source->getData();
				$formData['settings'] = unserialize($formData['settings']);
				Entity_Form_EmployeeApply::display($formData['settings'], $publication_id);

				$content .= '<script type="text/javascript"> if(carerix_jQuery) { carerix_jQuery(document).ready(function(e){ carerix_jQuery(".entry-meta").remove(); carerix_jQuery(".readmore").remove();  });  }   </script>';
			}
			//die();
		}

		if (isset($_GET['print'])) {
			ob_clean();
			global $post;
			echo '<style>';
			echo '@media print { .print { display: none; } };';
			echo '</style>';
			echo '<h1 class="cx_h1">' . $post->post_title . '</h1>';
			echo $content;
			echo '<a class="print" href="#" onclick="window.print();">Print</a>';
			die();
		}
		$job_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_JOB;

		if ($this->interfacer->wpml_is_active()) {
			$job_type .= '_' . $this->interfacer->get_default_site_language();
		}
		$author_id = isset($post->post_author) ? $post->post_author : null;
		if (!$author_id)
			return $content;

		$user = $this->interfacer->user_by_id($author_id);
		if ($user && $user->user_login == WP_Constants::CARERIX_USER_NICENAME) {
			remove_filter('the_content', 'wpautop');
		}

		$properties = $this->interfacer->entity_get_properties($post->ID);
		if (!empty($properties['managed_by']) && isset($properties['managed_by'][0]) && $properties['managed_by'][0] == WP_Constants::META_KEY_PREFIX) {
			remove_filter('the_content', 'wpautop');
		}

		if (isset($_GET['apply_mode'])) {
			$apply_mode = new Utils_Template('fragments/frontend/apply_mode');
			$apply_link = $this->interfacer->get_post_permalink(false, 'apply=true');
			$content = '<form method="post" enctype="multipart/form-data" action="' . $apply_link . '">' . $apply_mode->display(true) . '</form>';
			return $content;
		}

		$promotional_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_PROMOTIONAL;
		//$properties = $this->interfacer->entity_get_properties($post->ID);
		if (isset($properties[$job_type]) || isset($properties[$promotional_type]) || isset($properties['guid'])) {
			$jobFrontend = new Frontend_Job($properties);
			$content = $jobFrontend->display($content);
			return $content;
		}

		$article_type = WP_Constants::META_KEY_PREFIX . WP_Constants::ENTITY_TYPE_ARTICLE;
		if (isset($properties[$article_type])) {
			$articleFrontend = new Frontend_Article($properties);
			$content = $articleFrontend->display($content);
		}

		return $content;
	}

	public function registerWidgets() {
		register_widget('CarerixLoginWidget');
		register_widget('CarerixSearchWidget');
	}

	public function clear_taxonomies() {
		$taxonomies = array(
			WP_Constants::TAXONOMY_COUNTRY,
			WP_Constants::TAXONOMY_REGION,
			WP_Constants::TAXONOMY_WORKLOCATION,
			WP_Constants::TAXONOMY_EDUCATION,
			WP_Constants::TAXONOMY_FUNCTION,
			WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_1,
			WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_2,
			WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_3,
			WP_Constants::TAXONOMY_CANDIDATE_LAND,
			WP_Constants::TAXONOMY_CANDIDATE_REGION,
			WP_Constants::TAXONOMY_SECTOR,
			WP_Constants::TAXONOMY_INDUSTRY,
			WP_Constants::TAXONOMY_PROCEDURE,
			WP_Constants::TAXONOMY_BUSINESSLINE,
			WP_Constants::TAXONOMY_CONTRACTTYPE,
			WP_Constants::TAXONOMY_JOBGROUP,
			WP_Constants::TAXONOMY_HOURSPERWEEK,
			WP_Constants::TAXONOMY_MINSALARY,
			WP_Constants::TAXONOMY_MAXSALARY,
			WP_Constants::TAXONOMY_PUBIDLIST
		);

		$terms = get_terms($taxonomies, array('hide_empty' => false));
		foreach ($terms as $term) {
			$count = (int) $term->count;

			if ($count < 1) {
				$taxonomy = $term->taxonomy;
				$term_id = $term->term_id;
				wp_delete_term($term_id, $taxonomy);
			}
		}


		$remove_empty_cats = Environment_Settings::getSyncCleanEmptyCategories();
		if ($remove_empty_cats) {
			$taxonomies = array(
				'category'
			);

			$terms = get_terms($taxonomies, array('hide_empty' => false));

			foreach ($terms as $term) {
				$count = (int) $term->count;
				if ($count < 1) { // only process if there are no other sub-categories
					$taxonomy = $term->taxonomy;
					$term_id = $term->term_id;
					$managed_by_cx = count(get_term_meta($term->term_id, "managed_by")) ? true : false; // CX cats set by \WP_Interfacer::entity_link_category()
					if ($managed_by_cx) { // Only delete cats created by CX
						wp_delete_term($term_id, $taxonomy);
					}
				}
			}
		}
	}

	public function reset() {
		deactivate_plugins(plugin_basename(__FILE__));
		activate_plugin(plugin_basename(__FILE__));
	}

	public function disableCaching() {
		header( 'X-Cache-Enabled: False');
	}

	function activate_au() {
		$plugin_current_version = self::VERSION;
		$plugin_remote_path = self::REMOTE_PATH;
		$plugin_slug = plugin_basename( __FILE__ );
		new Environment_Autoupdate ( $plugin_current_version, $plugin_remote_path, $plugin_slug );
	}

	function set_apply_cookies() {
		$applySucceeded = Carerix\Cookie\Cookie::getCookie("applySucceeded");

		if ( $applySucceeded === true) {
			\Carerix\Cookie\Cookie::deleteCookie("cx_applysource");
			\Carerix\Cookie\Cookie::deleteCookie("cx_applytags");
			\Carerix\Cookie\Cookie::deleteCookie("applySucceeded");
		} else /*if (isset($_GET['apply']))*/ {
			if (isset($_GET['cx_applysource'])) {
				$cx_applysource = sanitize_text_field($_GET['cx_applysource']);
				\Carerix\Cookie\Cookie::setCookie("cx_applysource", $cx_applysource);
			}
			if (isset($_GET['cx_applytags'])) {
				$cx_applytags = sanitize_text_field($_GET['cx_applytags']);
				\Carerix\Cookie\Cookie::setCookie("cx_applytags", $cx_applytags);
			}
		}
	}

	function cx_verify_requirements() {
	    global $is_apache;
	    global $is_nginx;

	    if ($is_nginx) {
		    add_action( 'admin_notices', array($this, 'notice_unsupported_nginx') );
        } elseif (!$is_apache) {
		    add_action( 'admin_notices', array($this, 'notice_unsupported_webserver') );
	    }

		if (version_compare(phpversion(), CARERIX_PHP_MIN, '<')) {
			add_action( 'admin_notices', array($this, 'notice_unsupported_php_version') );
        }

		$memory_limit_raw = ini_get('memory_limit');
		$memory_limit_int = wp_convert_hr_to_bytes($memory_limit_raw);

		$memory_limit_mb  = -1; // -1 is unlimited
		if ($memory_limit_int !== -1) {
			$memory_limit_mb = $memory_limit_int / MB_IN_BYTES;
        }

        if ($memory_limit_mb > 0 && $memory_limit_mb < CARERIX_MEMORY_LIMIT_MIN) { // Insufficient PHP memory limit
	        add_action( 'admin_notices', array($this, 'notice_not_enough_memory') );
        }

        if ( Environment_Settings::getRestApiSync() !== "on" ) {
	        add_action( 'admin_notices', array($this, 'notice_recommend_rest_api_usage') );
        }
	}

	function notice_recommend_rest_api_usage() {
		?>
		<div class="notice notice-warning"><!-- is-dismissible -->
            <p><span style="font-size: 1.5em">&#9888;</span> <b>Carerix Plugin</b><br>In <a href="<?php echo admin_url("admin.php?page=carerix_general_page&tab=carerix_settings_page"); ?>">settings</a> you have not (yet) enabled <b>Synchronize job posts by REST-API</b>.<br>
                By enabling, you benefit from the support of more Carerix fields and functionalities (e.g. more detailed Google For Jobs integration).<br>
                <br>
                Warning: in the next release of this plugin the REST-API sync will be automatically activated. You won't have the option to disable it anymore.
            </p>
		</div>
		<?php
	}
	function notice_unsupported_php_version() {
		?>
		<div class="error">
            <p><span style="font-size: 1.5em">&#9888;</span> <b>Carerix Plugin</b><br>Version PHP <?php echo phpversion(); ?> is not supported and the plugin will not work correctly. Please <b>upgrade</b> to PHP <?php echo CARERIX_PHP_MIN; ?> or higher. Don't contact Carerix, but your hosting provider instead.</p>
		</div>
		<?php
	}
	function notice_unsupported_nginx() {
		?>
		<div class="error">
			<p><span style="font-size: 1.5em">&#9888;</span> <b>Carerix Plugin</b><br>Your hosting server NGINX is not supported. This will result in missing applications and vacancy synchronization errors. You need to run it on Apache. Don't contact Carerix, but your hosting provider instead.</p>
		</div>
		<?php
	}
	function notice_unsupported_webserver() {
		?>
		<div class="error">
			<p><span style="font-size: 1.5em">&#9888;</span> <b>Carerix Plugin</b><br>Your hosting server is not supported. You need to run it on Apache. Don't contact Carerix, but your hosting provider instead.</p>
		</div>
		<?php
	}
	function notice_not_enough_memory() {
		?>
		<div class="notice notice-error is-dismissible">
			<p><span style="font-size: 1.5em">&#9888;</span> <b>Carerix Plugin</b><br>Your hosting server has insufficient PHP memory. Please reconfigure the PHP settings (php_limit) to at least <?echo CARERIX_MEMORY_LIMIT_MIN; ?> MB. Don't contact Carerix, but your hosting provider instead.</p>
		</div>
		<?php
	}



	function cleanup_attachments_cron () {
		$cx_api_error = (new Admin_Diagnostics())->getCxApiConnectionStatus(); // 'null' means the connection is valid

		$uploadsDir = self::$TMPDIR . DIRECTORY_SEPARATOR . 'uploads' . DIRECTORY_SEPARATOR;
		$fields = scandir($uploadsDir);

		if ($fields) {
			foreach ($fields as $filename) {
				$fpath = $uploadsDir . $filename;
				if (is_file($fpath) && is_writable($fpath)) {
                    if ( isset($cx_api_error) ) { // api is down
                        @touch($fpath); // change modification time to present time to prevent uploads/attachments to be deleted. This way unprocessed (queued) applications can be auto-resubmitted to CX when the API-connection is restored
                    }
					$modificationTime = filemtime($fpath);
					if ($modificationTime < (time() - WP_Constants::CLEANUP_ATTACHMENTS_TIME)) {
						@unlink($fpath);
					}
				}
			}
		}
	}


	/**
	 * Filter hook carerix_search_action. This WP_Query filter can be used by developers to take the selections
	 * from the "Carerix Search Widget" into account in the "posts loop".
	 *
	 * Example:
	 * $wp_query = apply_filters('carerix_search_action', $wp_query);
	 * query_posts( $wp_query->query_vars );
	 *
	 *
	 * @param WP_Query $wp_query
	 *
	 * @return WP_Query
	 * @throws Exception
	 * @internal WP-147
	 */
	public function carerix_search_action(WP_Query $wp_query ) : WP_Query {
		$input_data = array();
		$input_data_post = filter_input_array(INPUT_POST, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);
		$input_data_get  = filter_input_array(INPUT_GET, FILTER_FLAG_STRIP_BACKTICK | FILTER_FLAG_ALLOW_OCTAL);

		if ($input_data_post) {
			$input_data += $input_data_post;
		}
		if ($input_data_get) {
			$input_data += $input_data_get;
		}

		if (!empty($input_data) && isset($input_data['carerix_search_action']) ) {
			$conditions = array(
				'start' => 0,
				'count' => 10000
			);
			if (isset($input_data['carerix_keyword']) && !empty($input_data['carerix_keyword'])) {
				$conditions['keyword'] = trim($input_data['carerix_keyword']);
			}

			if (isset($input_data['carerix_number']) && !empty($input_data['carerix_number'])) {
				$conditions['vacancyID'] = trim($input_data['carerix_number']);
			}

			if (isset($input_data['carerix_radius']) && !empty($input_data['carerix_radius'])) {
				$conditions['distance'] = trim($input_data['carerix_radius']);
			}

			if (isset($input_data['carerix_postalcode']) && !empty($input_data['carerix_postalcode'])) {
				$conditions['postalCode'] = trim($input_data['carerix_postalcode']);
			}

			if (isset($input_data['carerix_land']) && !empty($input_data['carerix_land'])) {
				$conditions['countryNode'] = trim($input_data['carerix_land']);
			}

			if(isset($input_data['function_group']) && !empty($input_data['function_group'])) {
				if (is_array($input_data['function_group'])) {
					$conditions['functionLevel1'] = array_map('trim', $input_data['function_group']);
				} else {
					$conditions['functionLevel1'] = trim($input_data['function_group']);
				}
			}

			if(isset($input_data['function']) && !empty($input_data['function'])) {
				if (is_array($input_data['function'])) {
					$conditions['functionLevel2'] = array_map('trim', $input_data['function']);
				} else {
					$conditions['functionLevel2'] = trim($input_data['function']);
				}
			}
			if(isset($input_data['worklevel']) && !empty($input_data['worklevel'])) {
				if (is_array($input_data['worklevel'])) {
					$conditions['workLevelNode'] = array_map('trim', $input_data['worklevel']);
				} else {
					$conditions['workLevelNode'] = trim($input_data['worklevel']);
				}
			}
			$conditions['show'] = array('eoID');

			$medium_codes = $this->interfacer->getMediumCodesFromRegisteredSources();
			$conditions['medium'] = $medium_codes;


			$publications = CRPublication::listBy($conditions);
//			$kept_publications = array();
			$keepPubs = array();


			if (count($publications) > 0) {
				$posts = $this->interfacer->get_posts_managed_by_carerix();
				foreach ($publications as $publication) {
					if(isset($posts[$publication->getId()] )) {
						array_push($keepPubs, $posts[$publication->getId()]);
					}
				}
			} else {
				$keepPubs[] = 0;
			}

			$wp_query->set('post__in', $keepPubs);
			$wp_query->set('is_search', true);
		}

        return $wp_query;
    }

	private function add_js_head_and_footer()
	{
		$labels = Utils_Labels::get();
		$fieldIsRequired = $labels['CxPortal.message.mandatory.mandatory'];
		$fieldFileSizeLarger = $labels['CxPortal.message.format.fileSizeLarger'];
		$fieldMandatoryEmail = $labels['CxPortal.message.mandatory.email'];

		$handle = $this->interfacer->register_script_asset('modernizr-custom.js', array( 'jquery' ), true);
		wp_add_inline_script($handle, "window.carerix_jQuery = jQuery.noConflict(); window.fieldIsRequiredMessage = '{$fieldIsRequired}'; window.fieldFileSizeLargerMessage = '{$fieldFileSizeLarger}'; window.fieldMandatoryEmailMessage = '{$fieldMandatoryEmail}';", 'before');
		$handle = $this->interfacer->register_script_asset('polyfiller.js', array( 'jquery' ), true);
		$shims_url = plugins_url('assets/scripts/shims/', __FILE__ );
		wp_add_inline_script($handle, "
if ( parseInt(window.carerix_jQuery.fn.jquery) == 1 ) {
  window.carerix_jQuery(document).ready(function() {
    window.carerix_jQuery.webshims.setOptions('waitReady', false);
    window.carerix_jQuery.webshims.setOptions('basePath', '{$shims_url}');
    window.carerix_jQuery.webshims.polyfill('forms forms-ext');
  });
}");
		$handle = $this->interfacer->register_script_asset('frontend.js', array( 'jquery' ), true);
		/**
		 * Store some vacancy details in the main js variable-object "window.cx" to make
		 * it accessible to 3rd party js tracking code.
		 */
		wp_add_inline_script($handle, "try {" . sprintf("cx.setPublicationID(%d); ", get_post_meta(get_the_ID(), 'guid', true))
			. sprintf("cx.setVacancyID(%d); ", get_post_meta(get_the_ID(), 'vacancyID', true))
			. sprintf("cx.setTitle(%s); ", json_encode(get_post_meta(get_the_ID(), 'title', true))) // this is NOT the original CX jobTitle but JobTitle + (optionally) Workcity
			. sprintf("cx.setGaClientID(\"%s\"); ", Environment_Settings::getGaClientId())
			. sprintf("cx.setGaClientIDWithTimestamp(\"%s\");", Environment_Settings::getGaClientIdWithTimestamp())
			. "} catch(err) { }"
		);

		wp_add_inline_script($handle, 'var ajaxurl = "' . admin_url('admin-ajax.php') . '"; ');
	}

}

try {
	global $wpdb;
	//$wpdb->show_errors();
	$carerix_app = new Carerix($wpdb);
	$carerix_app->run();
} catch (Exception $exception) {
	echo "<div class='error'><p>Carerix Platform Error : {$exception->getMessage()}</p></div>";
	;
	//Utils_Log::write("GENERAL_EXCEPTION", false, $exception);
}

/**
 * JS optimization exclude strings, as configured in admin page.
 *
 * @param $exclude: comma-seperated list of exclude strings
 * @return: comma-separated list of exclude strings
 */
function my_ao_override_jsexclude($exclude) {
	$plugin_dir = plugin_basename( plugin_dir_path( __FILE__ ) );
	return $exclude . ", /wp-content/plugins/{$plugin_dir}/";
}